/*
 * Copyright (c) 2007 Sun Microsystems, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/* STATUS
 * 
 * Coded sending but not tested
 * 
 * ToDo: 
 * - close connections on the stop
 * - add specific message code to the broadcasted packets
 * 
 */

package org.sunspotworld;

import org.sunspotworld.demo.util.*;

import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.Datagram;
import javax.microedition.io.DatagramConnection;

/**
 * Routines to control and read data from the SPOT's accelerometer and send them
 * via Radiogram packets to a display program running on a host computer.
 *<p>
 * The actual task scheduling is done by the PeriodicTask parent class.
 *<p>
 * Packets from the host application are received by the PacketReceiver class and
 * then dispatched to this class to respond to. The replies are sent back via the
 * PacketTransmitter class. The commands are defined in the PacketTypes class.
 * <p>
 * @author Ron Goldman<br>
 * Date: May 8, 2006, revised: August 1, 2007
 *
 * @see PeriodicTask
 * @see PacketHandler
 * @see PacketReceiver
 * @see PacketTransmitter
 * @see PacketTypes
 */
public class Broadcaster extends PeriodicTask 
implements PacketTypes{
    
    private StartApplication main;
    private int sampleInterval;     // in milliseconds
    
    private int currentSample = 0;
    private long startTime;
    
    // datagram objects
    DatagramConnection dgConnection = null;
    Datagram dg = null;

    
    /**
     * Create a new accelerometer controller.
     *
     * @param m reference to the main program getting commands from the host display
     * @param sampleInterval how often to sample the accelerometer, in milliseconds
     */
    public Broadcaster(StartApplication m, int sampleInterval) {
        super(3, sampleInterval, Thread.NORM_PRIORITY);
        this.sampleInterval = sampleInterval;
        main = m;
        System.out.println("Broadcaster Constructor Started");
        //start(); now the father of this object must start it and stop it
    }
    
    
    
    
    ///////////////////////////////////
    //
    // PeriodicTask overridden methods
    //
    ///////////////////////////////////
    
    /**
     * Routine called when task execution is about to start up.
     */
    public void starting() {
        System.out.println("Starting broadcaster sampler");
        
        // We create a DatagramConnection
        try {
            // The Connection is a broadcast so we specify it in the creation string
            dgConnection = (DatagramConnection) Connector.open("radiogram://broadcast:37");
            // Then, we ask for a datagram with the maximum size allowed
            dg = dgConnection.newDatagram(dgConnection.getMaximumLength());
        } catch (IOException ex) {
            System.out.println("Could not open radiogram broadcast connection");
            ex.printStackTrace();
            return;
        }


        
    }
    
    /**
     * Routine called when task execution is finished.
     */
    public void stopping() {
        
        try {
            dgConnection.close();
        }catch(IOException ex) {
            System.out.println("Could not open radiogram broadcast connection");
            ex.printStackTrace();
            return;
        }
        
        System.out.println("Stopping broadcaster sampler");
        
    }


    /**
     * Called once per task 
     */
    public void doTask() {
        try {
            dg.reset();
            dg.writeByte(BRCST_TO_BE_SENSED);
            dg.writeUTF("proximityBroadcast");
            dgConnection.send(dg);
            System.out.println("Broadcast is going through");
                

        }catch (IOException ex) {
               ex.printStackTrace(); 
        }
    }

    /** temporary fix until IService interface fixed */
    public void setName(String who){};
}


