/*
 * StartApplication.java
 *
 * Created on Apr 7, 2009 1:40:25 PM;
 */

package org.sunspotworld;


import com.sun.spot.peripheral.radio.RadioFactory;
import com.sun.spot.io.j2me.radiogram.*;
import com.sun.spot.io.j2me.radiostream.RadiostreamConnection;
import com.sun.spot.util.*;

import java.io.*;
import java.util.Hashtable;
import javax.microedition.io.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import org.sunspotworld.demo.util.PacketHandler;
import org.sunspotworld.demo.util.PacketReceiver;

/**
 * The startApp method of this class is called by the VM to start the
 * application.
 * 
 * The manifest specifies this class as MIDlet-1, which means it will
 * be selected for execution.
 */
public class StartApplication extends MIDlet 
implements  PacketHandler, PacketTypes{
    
    
    // Location as int
    private static final int HOME = 1;  
    private static final int OFFICE = 2;  
    private static final int CORRIDOR = 3;  
    private static final int OUTDOOR = 4;  
    
    
    // Duration context representation
    private static final int PASSBY = 1;  
    private static final int SHORT = 2;  
    private static final int MEDIUM = 3;  
    private static final int LARGE = 4;  
    // Duration treshholds
    private static final int PASSBY_MAX = 15;  
    private static final int SHORT_MAX = 50;  
    private static final int MEDIUM_MAX = 4*60;  
    
    
    private static final int ACCELEROMETER_SAMPLE_PERIOD = 250;  // read accelerometer every 250 msecs
    private static final int BROADCASTER_SAMPLE_PERIOD = 200;  // Send broadcasts every 200 msecs
    private AccelMonitorTest accelMonitor = null;
    private Broadcaster broadCasterInstance = null;

    private RadiogramConnection dgBroadcastReceiptConnection = null;
    private PacketReceiver broadcastRcvr = null;
    
    private Hashtable handshakesTable = null;

    private String myAddress = null;
    
    private int location = OUTDOOR; // default value is outdoor
    
    protected void startApp() throws MIDletStateChangeException {
        System.out.println("Hello, world");
        new BootloaderListener().start();   // monitor the USB (if connected) and recognize commands from host

        
        
        handshakesTable = new Hashtable();
        long ourAddr = RadioFactory.getRadioPolicyManager().getIEEEAddress();
        myAddress = IEEEAddress.toDottedHex(ourAddr);
        System.out.println("Our radio address = " + myAddress);

        initialize();
        run();
    }

    protected void pauseApp() {
        // This is not currently called by the Squawk VM
    }

    
        /////////////////////////////////////////////////////
    //
    // Lifecycle management - called from startApp() method
    //
    /////////////////////////////////////////////////////
    
    /**
     * Initialize any needed variables.
     */
    public void initialize() { 
        
        // create the monitor to handle accelerometer commands
        accelMonitor = new AccelMonitorTest(this, ACCELEROMETER_SAMPLE_PERIOD);
        System.out.println("Monitor should have been created");
        broadCasterInstance = new Broadcaster(this, BROADCASTER_SAMPLE_PERIOD);
        broadCasterInstance.stop();
        
        
                
        try {
            // PORT TEST, to remove
            //RadiogramConnection serverConn = (RadiogramConnection) Connector.open("radiogram://:");
            //byte port= 0;
            //port = serverConn.getLocalPort(); // this maybe will throw an exception as the 
                                           // port has not been allocated yet.
            //System.out.println("Port allocated equal to " + port);

            
            
            dgBroadcastReceiptConnection = (RadiogramConnection) Connector.open("radiogram://:37");        
            broadcastRcvr = new PacketReceiver(dgBroadcastReceiptConnection);        // set up thread to receive & dispatch commands
            broadcastRcvr.setServiceName("Broadcast Receiver is on!");

            broadcastRcvr.registerHandler(this, LOCATE_INFORMATION_ID);
            broadcastRcvr.registerHandler(this, BRCST_TO_BE_SENSED);
            broadcastRcvr.registerHandler(this, REPLY_TO_BROADCAST);
            broadcastRcvr.start();
        }
       catch (IOException ex) {
            System.out.println("Failed to open connection to host: " + ex);
            return;
        }
        
    }
    
    /**
     * Main application run loop.
     */
    public void run() {
        
        while (true) {
            Utils.sleep(30000);     // real work is done elsewhere; allow deep sleep while trying to connect
        }
    }


    
    
    /**
     * Called if the MIDlet is terminated by the system.
     * I.e. if startApp throws any exception other than MIDletStateChangeException,
     * if the isolate running the MIDlet is killed with Isolate.exit(), or
     * if VM.stopVM() is called.
     * 
     * It is not called if MIDlet.notifyDestroyed() was called.
     *
     * @param unconditional If true when this method is called, the MIDlet must
     *    cleanup and release all resources. If false the MIDlet may throw
     *    MIDletStateChangeException  to indicate it does not want to be destroyed
     *    at this time.
     */
    protected void destroyApp(boolean unconditional) throws MIDletStateChangeException {
    }
    
    
    /* event reception methods */
    
     public void receiveAccelInternalSignal(boolean movement) { 
        
        if (movement){
            broadCasterInstance.start();
        }
        else{
            broadCasterInstance.stop();
        }
         
         
    }

    
    //////////////////////////////////////////////////////////
    //
    // Packet Recieving handling - called from PacketReceiver class
    //
    //////////////////////////////////////////////////////////
    
    /**
     * Callback from PacketReceiver when a new command is received from the host.
     * Note that commands associated with the accelerometer are dispatched directly
     * to the AccelMonitor class's handlePacket() method.
     * 
     * @param type the command
     * @param pkt the radiogram with any other required information
     */
    public void handlePacket(byte type, Radiogram pkt) {
        int signalStrength;
        try {
            switch (type) {
                case LOCATE_INFORMATION_ID:
                    signalStrength = pkt.getRssi();
                    this.location = pkt.readInt();
                    System.out.println("Location packt received with power equal to "
                            + signalStrength + " and info equal to " 
                            + this.location);
                    
                    break;
                
                // sensing broadcast received
                case BRCST_TO_BE_SENSED:
                    signalStrength = pkt.getRssi();
                    System.out.println("The Broadcast was received from " + pkt.getAddress());
                    // TodO: add some checking to pkt.getAddress() being null
                    // ToDo: possibly add some MAC blacklisting
                    if(! handshakesTable.containsKey(pkt.getAddress()) ){// in case the handshake does not exist
                        System.out.println("Connection to be open");
                        RadiostreamConnection serverConn = (RadiostreamConnection) Connector.open("radiostream://" + pkt.getAddress() + ":" + TEST_PORT);
                        System.out.println("AAConnection should be open");
                        // port is now open
                        SocialHandshake sH = new SocialHandshake(pkt.getAddress(), myAddress,serverConn, true, this);
                        handshakesTable.put(sH.getDestMac(),sH);
                    }
                    
                    break;

                //     
                case REPLY_TO_BROADCAST:
                    signalStrength = pkt.getRssi();
                    byte connPort = pkt.readByte();
                    String targetUserId = "";
                    if( pkt.readByte() != UserIDTAG){
                        System.out.println("It was supposed to receive an ID tag but it didnt");
                    }
                    else{
                        targetUserId = pkt.readUTF();
                    }
                    
                    
                    System.out.println("Conection to be established on port " +  connPort);
                    // ToDo: possibly add some MAC blacklisting
                    if(! handshakesTable.containsKey(pkt.getAddress()) ){// in case the handshake does not exist
                        RadiostreamConnection serverConn = (RadiostreamConnection) Connector.open("radiostream://" +
                              pkt.getAddress() + ":" + connPort);
                        // this is the constructor for the client
                        SocialHandshake sH = new SocialHandshake(pkt.getAddress(), myAddress,serverConn, false,this,targetUserId); // as client
                        handshakesTable.put(sH.getDestMac(),sH);
                    }
                    
                    break;
                    
                    
            }
        } catch (IOException ex) {
            System.out.println("Exception: " +  ex.getMessage());
        }
    }
 
     public boolean removeHandshakeFromHash(String address) {
        if( address!=null && handshakesTable.containsKey(address) )
            handshakesTable.remove(address);
        else
            return false;
        
        return true;
     
     }
    
     public boolean addInDb(String destMac, long duration, int proximityAverage, String timestamp) {
        int trustArray[] = new int[3];
        this.assignTrust(this.location, duration, trustArray);
        
        // Todo send DB datagram
        System.out.println("Adding to db: dest " + destMac + ", duration " +  duration 
                 + ", timestamp " +  timestamp + ", location " +  this.location
                + ", public trust " +  trustArray[1] + ", private trust " +  trustArray[2]
                + " and professional trust " +  trustArray[0] );
        
        RadiogramConnection rCon = null;
        Datagram dg = null;

        try {
            rCon = (RadiogramConnection) Connector.open("radiogram://broadcast:" + DB_BRCAST_PORT);
            dg = rCon.newDatagram(rCon.getMaximumLength());
            dg.reset();
            dg.writeUTF(destMac);
            dg.writeUTF(timestamp);
            dg.writeUTF(locationDecoder(this.location));
            dg.writeLong(duration);
            dg.writeInt(trustArray[1]);
            dg.writeInt(trustArray[2]);
            dg.writeInt(trustArray[0]);
            rCon.send(dg);

        
        } catch (Exception e) {
            System.err.println("Caught " + e +
                    " expection");
            System.exit(1);
        }
        
        
        return true;
     
     }
     
     // trust array is defined as
     // trust [0] - professional
     // trust [1] - public
     // trust [2] - private
     
     public void assignTrust(int location, long duration, int[] trustArray){
        int contextDuration = 0;
        contextDuration = this.contextualizeDuration(duration);
        // Todo: test for -1, I mean if context duration is ok, test also location
        
        switch(location){
        
            case HOME:

                switch(contextDuration){
                    case PASSBY: 
                        trustArray[0] = 0;
                        trustArray[1] = 1;
                        trustArray[2] = 0;
                        break;
                    case SHORT:
                        trustArray[0] = 0;
                        trustArray[1] = 1;
                        trustArray[2] = 1;
                        break;
                    case MEDIUM:
                        trustArray[0] = 0;
                        trustArray[1] = 2;
                        trustArray[2] = 3;
                        break;
                    case LARGE:  
                        trustArray[0] = 0;
                        trustArray[1] = 5;
                        trustArray[2] = 5;                        
                        break;
                }// switch(contextDuration)                
            break;
            
            case OFFICE:

                switch(contextDuration){
                    case PASSBY: 
                        trustArray[0] = 1;
                        trustArray[1] = 0;
                        trustArray[2] = 0;
                        break;
                    case SHORT:
                        trustArray[0] = 2;
                        trustArray[1] = 1;
                        trustArray[2] = 0;
                        break;
                    case MEDIUM:
                        trustArray[0] = 4;
                        trustArray[1] = 2;
                        trustArray[2] = 0;
                        break;
                    case LARGE:  
                        trustArray[0] = 5;
                        trustArray[1] = 3;
                        trustArray[2] = 2;                        
                        break;
                }// switch(contextDuration)                
                
            break;
            
            case CORRIDOR:

                switch(contextDuration){
                    case PASSBY: 
                        trustArray[0] = 1;
                        trustArray[1] = 0;
                        trustArray[2] = 0;
                        break;
                    case SHORT:
                        trustArray[0] = 1;
                        trustArray[1] = 2;
                        trustArray[2] = 0;
                        break;
                    case MEDIUM:
                        trustArray[0] = 3;
                        trustArray[1] = 3;
                        trustArray[2] = 2;
                        break;
                    case LARGE:  // actually has not been defined
                        trustArray[0] = 0;
                        trustArray[1] = 0;
                        trustArray[2] = 0;                        
                        break;
                }// switch(contextDuration)                

                
            break;
            
            case OUTDOOR:

                switch(contextDuration){
                    case PASSBY: 
                        trustArray[0] = 0;
                        trustArray[1] = 0;
                        trustArray[2] = 0;
                        break;
                    case SHORT:
                        trustArray[0] = 0;
                        trustArray[1] = 1;
                        trustArray[2] = 0;
                        break;
                    case MEDIUM:
                        trustArray[0] = 0;
                        trustArray[1] = 2;
                        trustArray[2] = 1;
                        break;
                    case LARGE:  
                        trustArray[0] = 0;
                        trustArray[1] = 4;
                        trustArray[2] = 2;                        
                        break;
                }// switch(contextDuration)                
                
                
            break;

        
        
        }
        
        
     }

     public int contextualizeDuration(long duration){
        if(duration < 1)
            return -1;
        else{
            if (duration <= PASSBY_MAX)
                return PASSBY;
            if (duration > PASSBY_MAX && duration <= SHORT_MAX )
                return SHORT;
            if (duration > SHORT_MAX && duration <= MEDIUM_MAX )
                return MEDIUM;
            if (duration > MEDIUM_MAX)
                return LARGE;                
        }
        
        // else
        return -1;
        
     }

    public String locationDecoder (int location){
        switch(location){
            case HOME: return "HOME";
            case OFFICE: return "OFFICE";
            case CORRIDOR: return "CORRIDOR";
            case OUTDOOR: return "OUTDOOR";
            default: return "UNKNOWN";
        }
    }
}
